@@ -6,7 +6,7 @@ use Mojo::Util 'deprecated';
 use Carp 'croak';
 use strict;
 
-our $VERSION = '1.4';
+our $VERSION = '1.5';
 
 has providers => sub {
   return {
@@ -30,6 +30,7 @@ has providers => sub {
       authorize_url => "https://accounts.google.com/o/oauth2/auth?response_type=code",
       token_url     => "https://accounts.google.com/o/oauth2/token",
     },
+    mocked => {authorize_url => '/mocked/oauth/authorize', token_url => '/mocked/oauth/token', secret => 'fake_secret'},
   };
 };
 
@@ -52,6 +53,10 @@ sub register {
 
   $self->providers($providers);
 
+  if ($providers->{mocked}{key}) {
+    $self->_mock_interface($app);
+  }
+
   $app->helper('oauth2.auth_url'  => sub { $self->_get_authorize_url(@_) });
   $app->helper('oauth2.providers' => sub { $self->providers });
   $app->helper(
@@ -223,6 +228,43 @@ sub _process_response_error {
   $c->$cb($c->param('error_description') || $c->param('error'), undef);
 }
 
+sub _mock_interface {
+  my ($self, $app) = @_;
+  my $provider = $self->providers->{mocked};
+
+  $self->_ua->server->app($app);
+
+  $provider->{return_code}  ||= 'fake_code';
+  $provider->{return_token} ||= 'fake_token';
+
+  $app->routes->get(
+    $provider->{authorize_url} => sub {
+      my $c = shift;
+      if ($c->param('client_id') and $c->param('redirect_uri')) {
+        my $url = Mojo::URL->new($c->param('redirect_uri'));
+        $url->query->append(code => $provider->{return_code});
+        $c->render(text => $c->tag('a', href => $url, sub {'Connect'}));
+      }
+      else {
+        $c->render(text => "Invalid request\n", status => 400);
+      }
+    }
+  );
+
+  $app->routes->post(
+    $provider->{token_url} => sub {
+      my $c = shift;
+      if ($c->param('client_secret') and $c->param('redirect_uri') and $c->param('code')) {
+        my $qp = Mojo::Parameters->new(access_token => $provider->{return_token}, lifetime => 3600);
+        $c->render(text => $qp->to_string);
+      }
+      else {
+        $c->render(status => 404, text => 'FAIL OVERFLOW');
+      }
+    }
+  );
+}
+
 1;
 
 =head1 NAME
@@ -345,6 +387,45 @@ See also L<https://developers.google.com/+/quickstart/>.
 
 =back
 
+=head2 Testing
+
+THIS API IS EXPERIMENTAL AND CAN CHANGE WITHOUT NOTICE.
+
+To enable a "mocked" OAuth2 api, you need to give the special "mocked"
+provider a "key":
+
+  plugin "OAuth2" => { mocked => {key => 42} };
+
+The code above will add two new routes to your application:
+
+=over 4
+
+=item * GET /mocked/oauth/authorize
+
+This route is a web page which contains a link that takes you back to
+"redirect_uri", with a "code". The "code" default to "fake_code", but
+can be configured:
+
+  $c->app->oauth2->providers->{mocked}{return_code} = "...";
+
+The route it self can also be customized:
+
+  plugin "OAuth2" => { mocked => {authorize_url => '...'} };
+
+=item * POST /mocked/oauth/token
+
+This route is will return an "access_token" which will be the C<$token>
+variable in your L</oauth.get_token> callback. The default is "fake_token",
+but it can be configured:
+
+  $c->app->oauth2->providers->{mocked}{return_token} = "...";
+
+The route it self can also be customized:
+
+  plugin "OAuth2" => { mocked => {token_url => '...'} };
+
+=back
+
 =head1 HELPERS
 
 =head2 oauth2.auth_url
@@ -453,6 +534,18 @@ something like this:
     ...
   }
 
+=head1 ATTRIBUTES
+
+=head2 providers
+
+Holds a hash of provider information. See L<oauth2.providers>.
+
+=head1 METHODS
+
+=head2 register
+
+Will register this plugin in your application. See L</SYNOPSIS>.
+
 =head1 AUTHOR
 
 Marcus Ramberg - C<mramberg@cpan.org>
